@@ -174,6 +174,18 @@ class NoTestsError(Exception):
   """Raised when parameterized decorators do not generate any tests."""
 
 
+class DuplicateTestNameError(Exception):
+  """Raised when a parameterized test has the same test name multiple times."""
+
+  def __init__(self, test_class_name, new_test_name, original_test_name):
+    super(DuplicateTestNameError, self).__init__(
+        'Duplicate parameterized test name in {}: generated test name {!r} '
+        '(generated from {!r}) already exists. Consider using '
+        'named_parameters() to give your tests unique names and/or renaming '
+        'the conflicting test method.'.format(
+            test_class_name, new_test_name, original_test_name))
+
+
 def _clean_repr(obj):
   return _ADDR_RE.sub(r'<\1>', repr(obj))
 
@@ -197,7 +209,7 @@ def _format_parameter_list(testcase_params):
 class _ParameterizedTestIter(object):
   """Callable and iterable class for producing new test cases."""
 
-  def __init__(self, test_method, testcases, naming_type):
+  def __init__(self, test_method, testcases, naming_type, original_name=None):
     """Returns concrete test functions for a test and a list of parameters.
 
     The naming_type is used to determine the name of the concrete
@@ -207,13 +219,21 @@ class _ParameterizedTestIter(object):
 
     Args:
       test_method: The decorated test method.
-      testcases: (list of tuple/dict) A list of parameter
-                 tuples/dicts for individual test invocations.
+      testcases: (list of tuple/dict) A list of parameter tuples/dicts for
+          individual test invocations.
       naming_type: The test naming type, either _NAMED or _ARGUMENT_REPR.
+      original_name: The original test method name. When decorated on a test
+          method, None is passed to __init__ and test_method.__name__ is used.
+          Note test_method.__name__ might be different than the original defined
+          test method because of the use of other decorators. A more accurate
+          value is set by TestGeneratorMetaclass.__new__ later.
     """
     self._test_method = test_method
     self.testcases = testcases
     self._naming_type = naming_type
+    if original_name is None:
+      original_name = test_method.__name__
+    self._original_name = original_name
     self.__name__ = _ParameterizedTestIter.__name__
 
   def __call__(self, *args, **kwargs):
@@ -260,13 +280,14 @@ class _ParameterizedTestIter(object):
           raise RuntimeError(
               'Named tests must be passed a dict or non-string iterable.')
 
+        test_method_name = self._original_name
         # Support PEP-8 underscore style for test naming if used.
-        if (bound_param_test.__name__.startswith('test_')
+        if (test_method_name.startswith('test_')
             and testcase_name
             and not testcase_name.startswith('_')):
-          bound_param_test.__name__ += '_'
+          test_method_name += '_'
 
-        bound_param_test.__name__ += str(testcase_name)
+        bound_param_test.__name__ = test_method_name + str(testcase_name)
       elif naming_type is _ARGUMENT_REPR:
         # If it's a generator, convert it to a tuple and treat them as
         # parameters.
@@ -307,8 +328,8 @@ def _modify_class(class_object, testcases, naming_type):
       delattr(class_object, name)
       methods = {}
       _update_class_dict_for_param_test_case(
-          methods, test_method_ids, name,
-          _ParameterizedTestIter(obj, testcases, naming_type))
+          class_object.__name__, methods, test_method_ids, name,
+          _ParameterizedTestIter(obj, testcases, naming_type, name))
       for name, meth in six.iteritems(methods):
         setattr(class_object, name, meth)
 
@@ -414,10 +435,17 @@ class TestGeneratorMetaclass(type):
     for name, obj in six.iteritems(dct.copy()):
       if (name.startswith(unittest.TestLoader.testMethodPrefix) and
           _non_string_or_bytes_iterable(obj)):
+        if isinstance(obj, _ParameterizedTestIter):
+          # Update the original test method name so it's more accurate.
+          # The mismatch might happen when another decorator is used inside
+          # the parameterized decrators, and the inner decorator doesn't
+          # preserve its __name__.
+          # `obj` might be a generator, not _ParameterizedTestIter.
+          obj._original_name = name
         iterator = iter(obj)
         dct.pop(name)
         _update_class_dict_for_param_test_case(
-            dct, test_method_ids, name, iterator)
+            class_name, dct, test_method_ids, name, iterator)
     # If the base class is a subclass of parameterized.TestCase, inherit its
     # _test_method_ids too.
     for base in bases:
@@ -437,14 +465,18 @@ class TestGeneratorMetaclass(type):
 
 
 def _update_class_dict_for_param_test_case(
-    dct, test_method_ids, name, iterator):
+    test_class_name, dct, test_method_ids, name, iterator):
   """Adds individual test cases to a dictionary.
 
   Args:
+    test_class_name: The name of the class tests are added to.
     dct: The target dictionary.
     test_method_ids: The dictionary for mapping names to test IDs.
     name: The original name of the test case.
     iterator: The iterator generating the individual test cases.
+
+  Raises:
+    DuplicateTestNameError: Raised when a test name occurs multiple times.
   """
   for idx, func in enumerate(iterator):
     assert callable(func), 'Test generators must yield callables, got %r' % (
@@ -455,8 +487,10 @@ def _update_class_dict_for_param_test_case(
     else:
       original_name = name
       new_name = '%s%d' % (original_name, idx)
-    assert new_name not in dct, (
-        'Name of parameterized test case "%s" not unique' % (new_name,))
+
+    if new_name in dct:
+      raise DuplicateTestNameError(test_class_name, new_name, original_name)
+
     dct[new_name] = func
     test_method_id = original_name + getattr(func, '__x_extra_id__', '')
     assert test_method_id not in test_method_ids.values(), (
